@@ -2,11 +2,8 @@
 """Tests for mechanize.Browser."""
 
 import copy
-import httplib
-import mimetools
 import os
 import re
-import StringIO
 from io import BytesIO
 from unittest import TestCase
 
@@ -15,6 +12,10 @@ import mechanize._response
 import mechanize._testcase
 from mechanize._gzip import HTTPGzipProcessor, compress_readable_output
 from mechanize._response import test_html_response
+from mechanize.polyglot import (HTTPConnection, addinfourl, codepoint_to_chr,
+                                create_response_info, iteritems, unicode_type)
+
+em_dash = codepoint_to_chr(0x2014)
 
 
 # XXX these 'mock' classes are badly in need of simplification / removal
@@ -26,13 +27,13 @@ class MockMethod:
         self.action = action
 
     def __call__(self, *args):
-        return apply(self.handle, (self.meth_name, self.action) + args)
+        return self.handle(self.meth_name, self.action, *args)
 
 
 class MockHeaders(dict):
     def getheaders(self, name):
         name = name.lower()
-        return [v for k, v in self.iteritems() if name == k.lower()]
+        return [v for k, v in iteritems(self) if name == k.lower()]
 
     def __delitem__(self, k):
         kmap = {q.lower(): q for q in self}
@@ -46,7 +47,9 @@ class MockResponse:
 
     def __init__(self, url="http://example.com/", data=None, info=None):
         self.url = self._url = url
-        self.fp = StringIO.StringIO(data)
+        if isinstance(data, type(u'')):
+            data = data.encode('utf-8')
+        self.fp = BytesIO(data)
         if info is None:
             info = {}
         self._info = self._headers = MockHeaders(info)
@@ -167,10 +170,10 @@ class BrowserTests(TestCase):
         # Referer not added by .open()...
         req = mechanize.Request(url)
         b.open(req)
-        self.assert_(req.get_header("Referer") is None)
+        self.assertTrue(req.get_header("Referer") is None)
         # ...even if we're visiting a document
         b.open(req)
-        self.assert_(req.get_header("Referer") is None)
+        self.assertTrue(req.get_header("Referer") is None)
         # Referer added by .click_link() and .click()
         b.select_form("form1")
         req2 = b.click()
@@ -186,22 +189,20 @@ class BrowserTests(TestCase):
             req4.get_header("Referer"), "http://example.com/foo/bar.html")
         b.open(req4)
         req5 = b.click_link(name="apples")
-        self.assert_(not req5.has_header("Referer"))
+        self.assertTrue(not req5.has_header("Referer"))
         # Referer not added for non-http, non-https requests
         b.add_handler(make_mock_handler()([("blah_open", r)]))
         req6 = b.click_link(name="pears")
-        self.assert_(not req6.has_header("Referer"))
+        self.assertTrue(not req6.has_header("Referer"))
         # Referer not added when going from non-http, non-https URL
         b.open(req6)
         req7 = b.click_link(name="apples")
-        self.assert_(not req7.has_header("Referer"))
+        self.assertTrue(not req7.has_header("Referer"))
 
         # XXX Referer added for redirect
 
     def test_encoding(self):
         import mechanize
-        from StringIO import StringIO
-        import urllib
         # always take first encoding, since that's the one from the real HTTP
         # headers, rather than from HTTP-EQUIV
         b = mechanize.Browser()
@@ -212,8 +213,10 @@ class BrowserTests(TestCase):
             ("Content-Type: text/html; charset=UTF-8\r\n"
              "Content-Type: text/html; charset=KOI8-R\r\n\r\n", "UTF-8"),
         ]:
-            msg = mimetools.Message(StringIO(s))
-            r = urllib.addinfourl(StringIO(""), msg, "http://www.example.com/")
+            if not isinstance(s, bytes):
+                s = s.encode('ascii')
+            msg = create_response_info(BytesIO(s))
+            r = addinfourl(BytesIO(b""), msg, "http://www.example.com/")
             b.set_response(r)
             self.assertEqual(b.encoding(), ct)
 
@@ -238,26 +241,26 @@ class BrowserTests(TestCase):
         r1 = b.open("http://example.com/")
         self.assertRaises(mechanize.BrowserStateError, b.back)
         b.open("http://example.com/foo")
-        self.assert_(same_response(b.back(), r1))
+        self.assertTrue(same_response(b.back(), r1))
         r3 = b.open("http://example.com/bar")
         b.open("http://example.com/spam")
-        self.assert_(same_response(b.back(), r3))
-        self.assert_(same_response(b.back(), r1))
-        self.assertEquals(b.geturl(), "http://example.com/")
+        self.assertTrue(same_response(b.back(), r3))
+        self.assertTrue(same_response(b.back(), r1))
+        self.assertEqual(b.geturl(), "http://example.com/")
         self.assertRaises(mechanize.BrowserStateError, b.back)
         # reloading does a real HTTP fetch rather than using history cache
         r5 = b.reload()
-        self.assert_(not same_response(r5, r1))
+        self.assertTrue(not same_response(r5, r1))
         # .geturl() gets fed through to b.response
-        self.assertEquals(b.geturl(), "http://example.com/")
+        self.assertEqual(b.geturl(), "http://example.com/")
         # can go back n times
         b.open("spam")
-        self.assertEquals(b.geturl(), "http://example.com/spam")
+        self.assertEqual(b.geturl(), "http://example.com/spam")
         r7 = b.open("/spam")
-        self.assert_(same_response(b.response(), r7))
-        self.assertEquals(b.geturl(), "http://example.com/spam")
-        self.assert_(same_response(b.back(2), r5))
-        self.assertEquals(b.geturl(), "http://example.com/")
+        self.assertTrue(same_response(b.response(), r7))
+        self.assertEqual(b.geturl(), "http://example.com/spam")
+        self.assertTrue(same_response(b.back(2), r5))
+        self.assertEqual(b.geturl(), "http://example.com/")
         self.assertRaises(mechanize.BrowserStateError, b.back, 2)
         r8 = b.open("/spam")
 
@@ -266,7 +269,7 @@ class BrowserTests(TestCase):
         class Handler2(mechanize.BaseHandler):
             def https_open(self, request):
                 r = mechanize.HTTPError("https://example.com/bad", 503, "Oops",
-                                        MockHeaders(), StringIO.StringIO())
+                                        MockHeaders(), BytesIO())
                 return r
 
         b.add_handler(Handler2())
@@ -275,7 +278,7 @@ class BrowserTests(TestCase):
         self.assertEqual(b.response().geturl(), "https://example.com/bad")
         self.assertEqual(b.request.get_full_url(),
                          "https://example.com/badreq")
-        self.assert_(same_response(b.back(), r8))
+        self.assertTrue(same_response(b.back(), r8))
 
         # .close() should make use of Browser methods and attributes complain
         # noisily, since they should not be called after .close()
@@ -286,7 +289,7 @@ class BrowserTests(TestCase):
                      "clear_history set_cookie links forms viewing_html "
                      "encoding title select_form click submit click_link "
                      "follow_link find_link".split()):
-            self.assert_(getattr(b, attr) is None)
+            self.assertTrue(getattr(b, attr) is None)
 
     def test_reload_read_incomplete(self):
         import mechanize
@@ -325,7 +328,7 @@ class BrowserTests(TestCase):
         br.response().read()
         br.open('http://www.example.com/blah')
         br.back()
-        self.failIf(br.reloaded)
+        self.assertFalse(br.reloaded)
 
     def test_viewing_html(self):
         # XXX not testing multiple Content-Type headers
@@ -379,7 +382,7 @@ class BrowserTests(TestCase):
 
         b = TestBrowser()
 
-        self.assert_(b.response() is None)
+        self.assertTrue(b.response() is None)
 
         # To open a relative reference (often called a "relative URL"), you
         # have to have already opened a URL for it "to be relative to".
@@ -411,7 +414,7 @@ class BrowserTests(TestCase):
         b.add_handler(
             make_mock_handler()([("http_open", MockResponse(url, "", {}))]))
         r = b.open(url)
-        self.assert_(not b.viewing_html())
+        self.assertTrue(not b.viewing_html())
         test_state_error("click links forms title select_form".split())
         self.assertRaises(mechanize.BrowserStateError, b.submit, nr=0)
         self.assertRaises(mechanize.BrowserStateError, b.click_link, nr=0)
@@ -506,14 +509,15 @@ class BrowserTests(TestCase):
             r = b.open(url)
 
             Link = mechanize.Link
-            mdashx2 = u"\u2014".encode('utf-8') * 2
+            mdashx2 = em_dash.encode('utf-8') * 2
             qmdashx2 = clean_url(mdashx2, encoding)
             # base_url, url, text, tag, attrs
             exp = Link(
                 url, "http://example.com/foo/bar%s.html" % qmdashx2,
-                u"blah\u2014\u2014", "a",
-                [("href", u"http://example.com/foo/bar\u2014\u2014.html"),
-                 ("name", u"name0\u2014\u2014")])
+                unicode_type("blah") + (em_dash * 2), "a",
+                [("href", unicode_type("http://example.com/foo/bar%s.html") % (
+                    em_dash*2)),
+                 ("name", unicode_type("name0") + em_dash + em_dash)])
             # nr
             link = b.find_link()
             #             print
@@ -585,55 +589,56 @@ class BrowserTests(TestCase):
             self.assertEqual(got, expect)
         self.assertEqual(len(links), len(exp_links))
         # nr
-        l = b.find_link()
-        self.assertEqual(l.url, "http://example.com/foo/bar.html")
-        l = b.find_link(nr=1)
-        self.assertEqual(l.url, "spam")
+        lnk = b.find_link()
+        self.assertEqual(lnk.url, "http://example.com/foo/bar.html")
+        lnk = b.find_link(nr=1)
+        self.assertEqual(lnk.url, "spam")
         # text
-        l = b.find_link(text="yada yada")
-        self.assertEqual(l.url, "one")
+        lnk = b.find_link(text="yada yada")
+        self.assertEqual(lnk.url, "one")
         self.assertRaises(
             mechanize.LinkNotFoundError, b.find_link, text="da ya")
-        l = b.find_link(text_regex=re.compile("da ya"))
-        self.assertEqual(l.url, "one")
-        l = b.find_link(text_regex="da ya")
-        self.assertEqual(l.url, "one")
+        lnk = b.find_link(text_regex=re.compile("da ya"))
+        self.assertEqual(lnk.url, "one")
+        lnk = b.find_link(text_regex="da ya")
+        self.assertEqual(lnk.url, "one")
         # name
-        l = b.find_link(name="name3")
-        self.assertEqual(l.url, "one")
-        l = b.find_link(name_regex=re.compile("oo"))
-        self.assertEqual(l.url, "blah")
-        l = b.find_link(name_regex="oo")
-        self.assertEqual(l.url, "blah")
+        lnk = b.find_link(name="name3")
+        self.assertEqual(lnk.url, "one")
+        lnk = b.find_link(name_regex=re.compile("oo"))
+        self.assertEqual(lnk.url, "blah")
+        lnk = b.find_link(name_regex="oo")
+        self.assertEqual(lnk.url, "blah")
         # url
-        l = b.find_link(url="spam")
-        self.assertEqual(l.url, "spam")
-        l = b.find_link(url_regex=re.compile("pam"))
-        self.assertEqual(l.url, "spam")
-        l = b.find_link(url_regex="pam")
-        self.assertEqual(l.url, "spam")
+        lnk = b.find_link(url="spam")
+        self.assertEqual(lnk.url, "spam")
+        lnk = b.find_link(url_regex=re.compile("pam"))
+        self.assertEqual(lnk.url, "spam")
+        lnk = b.find_link(url_regex="pam")
+        self.assertEqual(lnk.url, "spam")
         # tag
-        l = b.find_link(tag="area")
-        self.assertEqual(l.url, "blah")
+        lnk = b.find_link(tag="area")
+        self.assertEqual(lnk.url, "blah")
         # predicate
-        l = b.find_link(
-            predicate=lambda l: dict(l.attrs).get("weird") == "stuff")
-        self.assertEqual(l.url, "two")
+        lnk = b.find_link(
+            predicate=lambda lnk: dict(lnk.attrs).get("weird") == "stuff")
+        self.assertEqual(lnk.url, "two")
         # combinations
-        l = b.find_link(name="pears", nr=1)
-        self.assertEqual(l.text, "rhubarb")
-        l = b.find_link(url="src", nr=0, name="name2")
-        self.assertEqual(l.tag, "iframe")
-        self.assertEqual(l.url, "src")
+        lnk = b.find_link(name="pears", nr=1)
+        self.assertEqual(lnk.text, "rhubarb")
+        lnk = b.find_link(url="src", nr=0, name="name2")
+        self.assertEqual(lnk.tag, "iframe")
+        self.assertEqual(lnk.url, "src")
         self.assertRaises(
             mechanize.LinkNotFoundError,
             b.find_link,
             url="src",
             nr=1,
             name="name2")
-        l = b.find_link(
-            tag="a", predicate=lambda l: dict(l.attrs).get("weird") == "stuff")
-        self.assertEqual(l.url, "two")
+        lnk = b.find_link(
+            tag="a", predicate=lambda lnk: dict(lnk.attrs).get(
+                "weird") == "stuff")
+        self.assertEqual(lnk.url, "two")
 
         # .links()
         self.assertEqual(
@@ -741,6 +746,7 @@ class BrowserTests(TestCase):
                       cbr._ua_handlers['_cookies'].cookiejar)
         self.assertIsNot(br.addheaders, cbr.addheaders)
         self.assertEqual(br.addheaders, cbr.addheaders)
+        self.assertIs(br.finalize_request_headers, cbr.finalize_request_headers)
         h = cbr._ua_handlers['_refresh']
         self.assertEqual((h.honor_time, h.max_time), (True, 237))
 
@@ -779,7 +785,7 @@ class ResponseTests(TestCase):
 
         br = TestBrowser()
         url = "http://example.com/"
-        html = """<html><body><a href="spam">click me</a></body></html>"""
+        html = b"""<html><body><a href="spam">click me</a></body></html>"""
         headers = {"content-type": "text/html"}
         r = response_seek_wrapper(MockResponse(url, html, headers))
         br.add_handler(make_mock_handler()([("http_open", r)]))
@@ -790,7 +796,7 @@ class ResponseTests(TestCase):
         self.assertEqual(copy.copy(r).read(), html)
         self.assertEqual(list(br.links())[0].url, "spam")
 
-        newhtml = """<html><body><a href="eggs">click me</a></body></html>"""
+        newhtml = b"""<html><body><a href="eggs">click me</a></body></html>"""
 
         r.set_data(newhtml)
         self.assertEqual(r.read(), newhtml)
@@ -807,13 +813,13 @@ class ResponseTests(TestCase):
     def test_select_form(self):
         from mechanize import _response
         br = TestBrowser()
-        fp = StringIO.StringIO('''<html>
+        fp = BytesIO(b'''<html>
             <form name="a"></form>
             <form name="b" data-ac="123"></form>
             <form name="c" class="x"></form>
             </html>''')
-        headers = mimetools.Message(
-            StringIO.StringIO("Content-type: text/html"))
+        headers = create_response_info(
+            BytesIO(b"Content-type: text/html"))
         response = _response.response_seek_wrapper(
             _response.closeable_response(fp, headers, "http://example.com/",
                                          200, "OK"))
@@ -834,9 +840,9 @@ class ResponseTests(TestCase):
         br = TestBrowser()
         self.assertEqual(str(br), "<TestBrowser (not visiting a URL)>")
 
-        fp = StringIO.StringIO('<html><form name="f"><input /></form></html>')
-        headers = mimetools.Message(
-            StringIO.StringIO("Content-type: text/html"))
+        fp = BytesIO(b'<html><form name="f"><input /></form></html>')
+        headers = create_response_info(
+            BytesIO(b"Content-type: text/html"))
         response = _response.response_seek_wrapper(
             _response.closeable_response(fp, headers, "http://example.com/",
                                          200, "OK"))
@@ -867,20 +873,27 @@ class HttplibTests(mechanize._testcase.TestCase):
 
         def getresponse(self_):
             class Response(object):
-                msg = mimetools.Message(StringIO.StringIO(""))
+                msg = create_response_info(BytesIO(b""))
                 status = 200
                 reason = "OK"
+                fp = BytesIO(b'')
 
                 def read(self__, sz=-1):
-                    return ""
+                    return b""
+
+                def close(self):
+                    self.fp = None
+
+                def readinto(self__, b):
+                    pass
 
             return Response()
 
-        self.monkey_patch(httplib.HTTPConnection, "putheader", putheader)
-        self.monkey_patch(httplib.HTTPConnection, "connect", do_nothing)
-        self.monkey_patch(httplib.HTTPConnection, "send", do_nothing)
-        self.monkey_patch(httplib.HTTPConnection, "close", do_nothing)
-        self.monkey_patch(httplib.HTTPConnection, "getresponse", getresponse)
+        self.monkey_patch(HTTPConnection, "putheader", putheader)
+        self.monkey_patch(HTTPConnection, "connect", do_nothing)
+        self.monkey_patch(HTTPConnection, "send", do_nothing)
+        self.monkey_patch(HTTPConnection, "close", do_nothing)
+        self.monkey_patch(HTTPConnection, "getresponse", getresponse)
 
     def test_add_host_header(self):
         headers = []
@@ -892,8 +905,193 @@ class HttplibTests(mechanize._testcase.TestCase):
         browser = self.make_browser()
         request = mechanize.Request("http://example.com/")
         browser.addheaders = [("Host", "myway.example.com")]
+        browser.finalize_request_headers = lambda req, headers: headers.__setitem__('Test-Header', 'Yay')
         browser.open(request)
         self.assertIn(("Host", "myway.example.com"), headers)
+        self.assertIn(("Test-Header", "Yay"), headers)
+
+    def test_misc_browser_tests(self):
+
+        class TestHttpHandler(mechanize.BaseHandler):
+            def http_open(self, request):
+                return mechanize._response.test_response(
+                        url=request.get_full_url())
+
+        class TestHttpBrowser(TestBrowser2):
+            handler_classes = TestBrowser2.handler_classes.copy()
+            handler_classes["http"] = TestHttpHandler
+            default_schemes = ["http"]
+
+        def response_impl(response):
+            return response.wrapped.fp.__class__.__name__
+
+        br = TestHttpBrowser()
+        r = br.open("http://example.com")
+        self.assertEqual('BytesIO', response_impl(r))
+        r2 = br.open("http://example.com")
+        self.assertEqual('BytesIO', response_impl(r2))
+        self.assertEqual('eofresponse', response_impl(r))
+        br.set_response(mechanize._response.test_response())
+        self.assertEqual('eofresponse', response_impl(r2))
+
+        br = TestHttpBrowser()
+        r = br.open("http://example.com")
+        r2 = mechanize._response.test_response(url="http://example.com/2")
+        self.assertEqual(response_impl(r2), 'BytesIO')
+        br.visit_response(r2)
+        self.assertEqual(response_impl(r), 'eofresponse')
+        self.assertEqual(br.geturl(), br.request.get_full_url())
+        self.assertEqual(br.geturl(), "http://example.com/2")
+        br.back()
+        self.assertEqual(br.geturl(), br.request.get_full_url())
+        self.assertEqual(br.geturl(), "http://example.com")
+
+        class ReloadCheckBrowser(TestHttpBrowser):
+            reloaded = False
+
+            def reload(self):
+                self.reloaded = True
+                return TestHttpBrowser.reload(self)
+
+        br = ReloadCheckBrowser()
+        old = br.open("http://example.com")
+        br.open("http://example.com/2")
+        new = br.back()
+        self.assertTrue(br.reloaded)
+        self.assertIsNot(new.wrapped, old.wrapped)
+
+        br = TestBrowser2()
+        self.assertRaises(ValueError, br.set_response, 'blah')
+        self.assertRaises(ValueError, br.set_response, BytesIO())
+        self.assertRaises(mechanize.URLError, br.open, "http://example.com")
+        self.assertRaises(mechanize.URLError, br.reload)
+
+        br = TestBrowser2()
+        br.add_handler(make_mock_handler(mechanize._response.test_response)([("http_open", None)]))
+
+        self.assertIsNone(br.response())
+        self.assertEqual(len(br._history._history), 0)
+        br.open("http://example.com/1")
+        self.assertIsNotNone(br.response())
+        self.assertEqual(len(br._history._history), 0)
+        br.clear_history()
+        self.assertIsNotNone(br.response())
+        self.assertEqual(len(br._history._history), 0)
+        br.open("http://example.com/2")
+        br.response() is not None
+        self.assertIsNotNone(br.response())
+        self.assertEqual(len(br._history._history), 1)
+        br.clear_history()
+        self.assertIsNotNone(br.response())
+        self.assertEqual(len(br._history._history), 0)
+
+        from test.test_urllib2 import MockHTTPHandler
+
+        def make_browser_with_redirect():
+            br = TestBrowser2()
+            hh = MockHTTPHandler(302, "Location: http://example.com/\r\n\r\n")
+            br.add_handler(hh)
+            br.add_handler(mechanize.HTTPRedirectHandler())
+            return br
+
+        def test_state(br):
+            self.assertIsNone(br.request)
+            self.assertIsNone(br.response())
+            self.assertRaises(mechanize.BrowserStateError, br.back)
+
+        br = make_browser_with_redirect()
+        test_state(br)
+        req = mechanize.Request("http://example.com")
+        req.visit = False
+        br.open(req)
+        test_state(br)
+
+        br = make_browser_with_redirect()
+        test_state(br)
+
+        req = mechanize.Request("http://example.com")
+        self.assertIsNone(req.visit)
+        br.open_novisit(req)
+        test_state(br)
+        self.assertFalse(req.visit)
+
+        def test_one_visit(handlers):
+            br = TestBrowser2()
+            for handler in handlers:
+                br.add_handler(handler)
+            req = mechanize.Request("http://example.com")
+            req.visit = True
+            br.open(req)
+            return br
+
+        def test_state(br):
+            # XXX the _history._history check is needed because of the weird
+            # throwing-away of history entries by .back() where response is
+            # None, which makes the .back() check insufficient to tell if a
+            # history entry was .add()ed.  I don't want to change this until
+            # post-stable.
+            self.assertTrue(br.response())
+            self.assertTrue(br.request)
+            self.assertEqual(len(br._history._history), 0)
+            self.assertRaises(mechanize.BrowserStateError, br.back)
+
+        from test.test_urllib2 import HTTPRedirectHandler
+        hh = MockHTTPHandler(302, "Location: http://example.com/\r\n\r\n")
+        br = test_one_visit([hh, HTTPRedirectHandler()])
+        test_state(br)
+
+        class MockPasswordManager:
+
+            def add_password(self, realm, uri, user, password):
+                pass
+
+            def find_user_password(self, realm, authuri):
+                return '', ''
+
+        ah = mechanize.HTTPBasicAuthHandler(MockPasswordManager())
+        hh = MockHTTPHandler(
+            401, 'WWW-Authenticate: Basic realm="realm"\r\n\r\n')
+        test_state(test_one_visit([hh, ah]))
+        ph = mechanize.ProxyHandler(dict(http="proxy.example.com:3128"))
+        ah = mechanize.ProxyBasicAuthHandler(MockPasswordManager())
+        hh = MockHTTPHandler(
+            407, 'Proxy-Authenticate: Basic realm="realm"\r\n\r\n')
+        test_state(test_one_visit([ph, hh, ah]))
+
+        from mechanize._response import test_response
+        br = TestBrowser2()
+        html = b"""\
+        <html><body>
+        <input type="text" name="a" />
+        <form><input type="text" name="b" /></form>
+        </body></html>
+        """
+        response = test_response(html, headers=[("Content-type", "text/html")])
+        self.assertRaises(mechanize.BrowserStateError, br.global_form)
+        br.set_response(response)
+        self.assertEqual(str(br.global_form().find_control(nr=0).name), 'a')
+        self.assertEqual(len(list(br.forms())), 1)
+        self.assertEqual(str(next(iter(br.forms())).find_control(nr=0).name), 'b')
+
+        from mechanize._response import test_html_response
+        br = TestBrowser2()
+        br.visit_response(test_html_response(b"""\
+<html><head><title></title></head><body>
+<input type="text" name="a" value="b"></input>
+<form>
+    <input type="text" name="p" value="q"></input>
+</form>
+</body></html>"""))
+
+        def has_a(form):
+            try:
+                form.find_control(name="a")
+            except mechanize.ControlNotFoundError:
+                return False
+            else:
+                return True
+        br.select_form(predicate=has_a)
+        self.assertEqual(str(br.form.find_control(name="a").value), 'b')
 
 
 if __name__ == "__main__":
