@@ -10,15 +10,13 @@ to mechanize.
 # CacheFTPHandler (hard to write)
 # parse_keqv_list, parse_http_list
 
-import StringIO
-import httplib
 import os
 import sys
 import unittest
+from io import BytesIO
 
 import mechanize
 
-from mechanize._http import parse_head
 from mechanize._response import test_response
 from mechanize import HTTPRedirectHandler, \
     HTTPEquivProcessor, HTTPRefreshProcessor, \
@@ -33,17 +31,13 @@ import mechanize._sockettimeout as _sockettimeout
 import mechanize._testcase
 import mechanize._urllib2_fork
 from mechanize._mechanize import sanepathname2url
+from mechanize.polyglot import create_response_info, iteritems
 
-## from logging import getLogger, DEBUG
-## l = getLogger("mechanize")
+# from logging import getLogger, DEBUG
+# l = getLogger("mechanize")
 # l.setLevel(DEBUG)
 
 
-class AlwaysEqual:
-    def __cmp__(self, other):
-        return 0
-
-
 class TrivialTests(mechanize._testcase.TestCase):
     def test_trivial(self):
         # A couple trivial tests
@@ -51,22 +45,16 @@ class TrivialTests(mechanize._testcase.TestCase):
         self.assertRaises(ValueError, mechanize.urlopen, 'bogus url')
 
         fname = os.path.join(self.make_temp_dir(), "test.txt")
-        write_file(fname, "data")
-        if fname[1:2] == ":":
-            fname = fname[2:]
-        # And more hacking to get it to work on MacOS. This assumes
-        # urllib.pathname2url works, unfortunately...
-        if os.name == 'mac':
-            fname = '/' + fname.replace(':', '/')
-        elif os.name == 'riscos':
-            import string
-            fname = os.expand(fname)
-            fname = fname.translate(string.maketrans("/.", "./"))
-
-        file_url = "file://%s" % sanepathname2url(fname)
-        f = mechanize.urlopen(file_url)
-
-        buf = f.read()
+        data = b'data'
+        write_file(fname, data)
+        if os.sep == '\\':
+            fname = '/' + fname
+        file_url = "file://" + fname
+        try:
+            f = mechanize.urlopen(file_url)
+        except Exception as e:
+            raise ValueError('Failed to open URL: {} for fname: {} with error: {}'.format(file_url, fname, e))
+        self.assertEqual(f.read(), data)
         f.close()
 
     def test_parse_http_list(self):
@@ -77,7 +65,7 @@ class TrivialTests(mechanize._testcase.TestCase):
                    'h']), ('a="b\\"c", d="e\\,f", g="h\\\\i"',
                            ['a="b"c"', 'd="e,f"', 'g="h\\i"'])]
         for string, list in tests:
-            self.assertEquals(
+            self.assertEqual(
                 mechanize._urllib2_fork.parse_http_list(string), list)
 
 
@@ -260,6 +248,7 @@ def test_password_manager_default_port(self):
 
 class MockOpener:
     addheaders = []
+    finalize_request_headers = None
 
     def open(self,
              req,
@@ -281,6 +270,10 @@ class MockFile:
     def close(self):
         pass
 
+    def __iter__(self):
+        for i in ():
+            yield i
+
 
 def http_message(mapping):
     """
@@ -289,16 +282,21 @@ def http_message(mapping):
 
     """
     f = []
-    for kv in mapping.items():
+    for kv in iteritems(mapping):
         f.append("%s: %s" % kv)
     f.append("")
-    msg = httplib.HTTPMessage(StringIO.StringIO("\r\n".join(f)))
+    msg = "\r\n".join(f)
+    if not isinstance(msg, bytes):
+        msg = msg.encode('iso-8859-1')
+    msg = create_response_info(BytesIO(msg))
     return msg
 
 
-class MockResponse(StringIO.StringIO):
+class MockResponse(BytesIO):
     def __init__(self, code, msg, headers, data, url=None):
-        StringIO.StringIO.__init__(self, data)
+        if not isinstance(data, bytes):
+            data = data.encode('utf-8')
+        BytesIO.__init__(self, data)
         self.code, self.msg, self.headers, self.url = code, msg, headers, url
 
     def info(self):
@@ -439,14 +437,13 @@ class MockHTTPHandler(mechanize.BaseHandler):
         self.requests = []
 
     def http_open(self, req):
-        import mimetools
         import copy
-        from StringIO import StringIO
         self.requests.append(copy.deepcopy(req))
         if self._count == 0:
             self._count = self._count + 1
             name = "Not important"
-            msg = mimetools.Message(StringIO(self.headers))
+            msg = create_response_info(BytesIO(
+                self.headers.encode('iso-8859-1')))
             return self.parent.error("http", req,
                                      test_response(), self.code, name, msg)
         else:
@@ -462,7 +459,13 @@ class MockHTTPResponse:
         self.reason = reason
 
     def read(self):
-        return ''
+        return b''
+
+    def readinto(self, b):
+        pass
+
+    def close(self):
+        self.fp = None
 
 
 class MockHTTPClass:
@@ -491,7 +494,7 @@ class MockHTTPClass:
     def request(self, method, url, body=None, headers={}):
         self.method = method
         self.selector = url
-        self.req_headers += headers.items()
+        self.req_headers += list(iteritems(headers))
         self.req_headers.sort()
         if body:
             self.data = body
@@ -513,19 +516,6 @@ class MockHTTPSHandler(AbstractHTTPHandler):
         return self.do_open(self.httpconn, req)
 
 
-class MockPasswordManager:
-    def add_password(self, realm, uri, user, password):
-        self.realm = realm
-        self.url = uri
-        self.user = user
-        self.password = password
-
-    def find_user_password(self, realm, authuri):
-        self.target_realm = realm
-        self.target_url = authuri
-        return self.user, self.password
-
-
 class OpenerDirectorTests(unittest.TestCase):
     def test_add_non_handler(self):
         class NonHandler(object):
@@ -550,7 +540,7 @@ class OpenerDirectorTests(unittest.TestCase):
             [("do_open", "return self"), ("proxy_open", "return self")],
             [("redirect_request", "return self")],
         ]
-        handlers = add_ordered_mock_handlers(o, meth_spec)
+        add_ordered_mock_handlers(o, meth_spec)
         o.add_handler(mechanize.UnknownHandler())
         for scheme in "do", "proxy", "redirect":
             self.assertRaises(URLError, o.open, scheme + "://example.com/")
@@ -623,7 +613,7 @@ class OpenerDirectorTests(unittest.TestCase):
         o.add_handler(p)
         o._maybe_reindex_handlers()
         self.assertEqual(o.handle_open, {"http": [h]})
-        self.assertEqual(len(o.process_response.keys()), 1)
+        self.assertEqual(len(list(o.process_response.keys())), 1)
         self.assertEqual(list(o.process_response["http"]), [p])
         self.assertEqual(list(o._any_response), [p])
         self.assertEqual(o.handlers, [h, p])
@@ -644,7 +634,7 @@ class OpenerDirectorTests(unittest.TestCase):
             handlers.append(h)
             o.add_handler(h)
 
-        r = o.open("http://example.com/")
+        o.open("http://example.com/")
         # handlers called in reverse order, thanks to their sort order
         self.assertEqual(o.calls[0][0], handlers[1])
         self.assertEqual(o.calls[1][0], handlers[0])
@@ -680,14 +670,18 @@ class OpenerDirectorTests(unittest.TestCase):
         handlers = add_ordered_mock_handlers(o, meth_spec)
 
         req = Request("http://example.com/")
-        r = o.open(req)
+        o.open(req)
         assert len(o.calls) == 2
+        ignore = object()
         calls = [(handlers[0], "http_open", (req, )), (
-            handlers[2], "http_error_302", (req, AlwaysEqual(), 302, "", {}))]
+            handlers[2], "http_error_302", (req, ignore, 302, "", {}))]
         for expected, got in zip(calls, o.calls):
             handler, method_name, args = expected
             self.assertEqual((handler, method_name), got[:2])
-            self.assertEqual(args, got[2])
+            self.assertEqual(len(args), len(got[2]))
+            for a, b in zip(args, got[2]):
+                if a is not ignore:
+                    self.assertEqual(a, b)
 
     def test_http_error_raised(self):
         # should get an HTTPError if an HTTP handler raises a non-200 response
@@ -717,7 +711,7 @@ class OpenerDirectorTests(unittest.TestCase):
         handlers = add_ordered_mock_handlers(o, meth_spec)
 
         req = Request("http://example.com/")
-        r = o.open(req)
+        o.open(req)
         # processor methods are called on *all* handlers that define them,
         # not just the first handler that handles the request
         calls = [(handlers[0], "http_request"), (handlers[1], "http_request"),
@@ -758,7 +752,7 @@ class OpenerDirectorTests(unittest.TestCase):
         for scheme in ["http", "ftp"]:
             o.calls = []
             req = Request("%s://example.com/" % scheme)
-            r = o.open(req)
+            o.open(req)
 
             calls = [
                 (handler, "any_request"),
@@ -771,18 +765,18 @@ class OpenerDirectorTests(unittest.TestCase):
                     enumerate(zip(o.calls, calls))):
                 if i < 2:
                     # *_request
-                    self.assert_((handler, name) == calls)
-                    self.assert_(len(args) == 1)
-                    self.assert_(isinstance(args[0], Request))
+                    self.assertTrue((handler, name) == calls)
+                    self.assertTrue(len(args) == 1)
+                    self.assertTrue(isinstance(args[0], Request))
                 else:
                     # *_response
-                    self.assert_((handler, name) == calls)
-                    self.assert_(len(args) == 2)
-                    self.assert_(isinstance(args[0], Request))
+                    self.assertTrue((handler, name) == calls)
+                    self.assertTrue(len(args) == 2)
+                    self.assertTrue(isinstance(args[0], Request))
                     # response from opener.open is None, because there's no
                     # handler that defines http_open to handle it
-                    self.assert_(args[1] is None or
-                                 isinstance(args[1], MockResponse))
+                    self.assertTrue(args[1] is None or isinstance(
+                        args[1], MockResponse))
 
 
 class MockRobotFileParserClass:
@@ -835,7 +829,9 @@ class HandlerTests(mechanize._testcase.TestCase):
 
             def retrfile(self, filename, filetype):
                 self.filename, self.filetype = filename, filetype
-                return StringIO.StringIO(self.data), len(self.data)
+                data = self.data if isinstance(
+                        self.data, bytes) else self.data.encode('utf-8')
+                return BytesIO(data), len(self.data)
 
         class NullFTPHandler(mechanize.FTPHandler):
             def __init__(self, data):
@@ -853,7 +849,7 @@ class HandlerTests(mechanize._testcase.TestCase):
         import socket
         data = "rheum rhaponicum"
         h = NullFTPHandler(data)
-        o = h.parent = MockOpener()
+        h.parent = MockOpener()
 
         for url, host, port, type_, dirs, timeout, filename, mimetype in [
             ("ftp://localhost/foo/bar/baz.html", "localhost", ftplib.FTP_PORT,
@@ -873,7 +869,7 @@ class HandlerTests(mechanize._testcase.TestCase):
             self.assertEqual(h.port, port)
             self.assertEqual(h.dirs, dirs)
             if sys.version_info >= (2, 6):
-                self.assertEquals(h.timeout, timeout)
+                self.assertEqual(h.timeout, timeout)
             self.assertEqual(h.ftpwrapper.filename, filename)
             self.assertEqual(h.ftpwrapper.filetype, type_)
             headers = r.info()
@@ -881,14 +877,14 @@ class HandlerTests(mechanize._testcase.TestCase):
             self.assertEqual(int(headers["Content-length"]), len(data))
 
     def test_file(self):
-        import rfc822
+        from email.utils import formatdate
         import socket
         h = mechanize.FileHandler()
         o = h.parent = MockOpener()
 
         temp_file = os.path.join(self.make_temp_dir(), "test.txt")
         urlpath = sanepathname2url(os.path.abspath(temp_file))
-        towrite = "hello, world\n"
+        towrite = b"hello, world\n"
         try:
             fqdn = socket.gethostbyname(socket.gethostname())
         except socket.gaierror:
@@ -903,11 +899,11 @@ class HandlerTests(mechanize._testcase.TestCase):
             try:
                 data = r.read()
                 headers = r.info()
-                newurl = r.geturl()
+                r.geturl()
             finally:
                 r.close()
             stats = os.stat(temp_file)
-            modified = rfc822.formatdate(stats.st_mtime)
+            modified = formatdate(stats.st_mtime, usegmt=True)
             self.assertEqual(data, towrite)
             self.assertEqual(headers["Content-type"], "text/plain")
             self.assertEqual(headers["Content-length"], "13")
@@ -919,8 +915,8 @@ class HandlerTests(mechanize._testcase.TestCase):
                 "file://%s:80%s/%s" % (socket.gethostbyname('localhost'),
                                        sanepathname2url(os.getcwd()),
                                        temp_file),
-                "file://somerandomhost.ontheinternet.com%s/%s" %
-            (sanepathname2url(os.getcwd()), temp_file),
+                "file://somerandomhost.ontheinternet.com%s/%s" % (
+                    sanepathname2url(os.getcwd()), temp_file),
         ]:
             write_file(temp_file, towrite)
             self.assertRaises(mechanize.URLError, h.file_open, Request(url))
@@ -959,7 +955,13 @@ class HandlerTests(mechanize._testcase.TestCase):
                 self.reason = reason
 
             def read(self):
-                return ''
+                return b''
+
+            def readinto(self, b):
+                pass
+
+            def close(self):
+                self.fp = None
 
         class MockHTTPClass:
             def __init__(self):
@@ -980,8 +982,7 @@ class HandlerTests(mechanize._testcase.TestCase):
             def request(self, method, url, body=None, headers={}):
                 self.method = method
                 self.selector = url
-                self.req_headers += headers.items()
-                self.req_headers.sort()
+                self.req_headers += list(iteritems(headers))
                 if body:
                     self.data = body
                 if self.raise_on_endheaders:
@@ -997,6 +998,7 @@ class HandlerTests(mechanize._testcase.TestCase):
         url = "http://example.com/"
         for method, data in [("GET", None), ("POST", "blah")]:
             req = Request(url, data, {"Foo": "bar"})
+            req.add_header('Order', '1')
             req.add_unredirected_header("Spam", "eggs")
             http = MockHTTPClass()
             r = h.do_open(http, req)
@@ -1009,16 +1011,18 @@ class HandlerTests(mechanize._testcase.TestCase):
             r.code, r.msg == 200, "OK"  # added from MockHTTPClass.getreply()
             hdrs = r.info()
             hdrs.get
-            hdrs.has_key  # r.info() gives dict from .getreply()
+            hdrs.__contains__  # r.info() gives dict from .getreply()
             self.assertEqual(r.geturl(), url)
 
             self.assertEqual(http.host, "example.com")
             self.assertEqual(http.level, 0)
             self.assertEqual(http.method, method)
             self.assertEqual(http.selector, "/")
-            self.assertEqual(http.req_headers,
-                             [("Connection", "close"), ("Foo", "bar"),
-                              ("Spam", "eggs")])
+            self.assertEqual(
+                http.req_headers,
+                [('Foo', 'bar'), ('Order', '1'),
+                    ('Spam', 'eggs'), ('Connection', 'close')]
+            )
             self.assertEqual(http.data, data)
 
         # check socket.error converted to URLError
@@ -1030,7 +1034,7 @@ class HandlerTests(mechanize._testcase.TestCase):
         for data in "", None:  # POST, GET
             req = Request("http://example.com/", data)
             r = MockResponse(200, "OK", {}, "")
-            newreq = h.do_request_(req)
+            h.do_request_(req)
             if data is None:  # GET
                 self.assertTrue("Content-length" not in req.unredirected_hdrs)
                 self.assertTrue("Content-type" not in req.unredirected_hdrs)
@@ -1047,7 +1051,7 @@ class HandlerTests(mechanize._testcase.TestCase):
             req.add_unredirected_header("Content-Type", "bar")
             req.add_unredirected_header("Host", "baz")
             req.add_unredirected_header("Spam", "foo")
-            newreq = h.do_request_(req)
+            h.do_request_(req)
             self.assertEqual(req.unredirected_hdrs["Content-Length"], "foo")
             self.assertEqual(req.unredirected_hdrs["Content-Type"], "bar")
             self.assertEqual(req.unredirected_hdrs["Host"], "baz")
@@ -1058,7 +1062,7 @@ class HandlerTests(mechanize._testcase.TestCase):
         # doesn't break anything Previously, a double slash directly after the
         # host could cause incorrect parsing of the url
         h = AbstractHTTPHandler()
-        o = h.parent = MockOpener()
+        h.parent = MockOpener()
 
         data = ""
         ds_urls = [
@@ -1103,28 +1107,28 @@ class HandlerTests(mechanize._testcase.TestCase):
         r = mechanize._response.test_response(code=502, msg="Bad gateway")
         self.assertTrue(h.http_response(req, r) is None)
         self.assertEqual(o.proto, "http")  # o.error called
-        self.assertEqual(o.args, (req, r, 502, "Bad gateway", AlwaysEqual()))
+        self.assertEqual(o.args[:4], (req, r, 502, "Bad gateway"))
 
     def test_referer(self):
         h = HTTPRefererProcessor()
-        o = h.parent = MockOpener()
+        h.parent = MockOpener()
 
         # normal case
         url = "http://example.com/"
         req = Request(url)
         r = MockResponse(200, "OK", {}, "", url)
         newr = h.http_response(req, r)
-        self.assert_(r is newr)
-        self.assert_(h.referer == url)
+        self.assertTrue(r is newr)
+        self.assertTrue(h.referer == url)
         newreq = h.http_request(req)
-        self.assert_(req is newreq)
-        self.assert_(req.unredirected_hdrs["Referer"] == url)
+        self.assertTrue(req is newreq)
+        self.assertTrue(req.unredirected_hdrs["Referer"] == url)
         # don't clobber existing Referer
         ref = "http://set.by.user.com/"
         req.add_unredirected_header("Referer", ref)
         newreq = h.http_request(req)
-        self.assert_(req is newreq)
-        self.assert_(req.unredirected_hdrs["Referer"] == ref)
+        self.assertTrue(req is newreq)
+        self.assertTrue(req.unredirected_hdrs["Referer"] == ref)
 
     def test_raise_http_errors(self):
         # HTTPDefaultErrorHandler should raise HTTPError if no error handler
@@ -1142,19 +1146,19 @@ class HandlerTests(mechanize._testcase.TestCase):
         try:
             h.http_error_default(request, response, code, msg, response.info())
         except mechanize.HTTPError as exc:
-            self.assert_(exc is not response)
-            self.assert_(exc.fp is response)
+            self.assertTrue(exc is not response)
+            self.assertTrue(exc.fp is response)
         else:
-            self.assert_(False)
+            self.assertTrue(False)
 
         # case 2. response object is already an HTTPError, so just re-raise it
         error = mechanize.HTTPError(url, code, msg, "fake headers", response)
         try:
             h.http_error_default(request, error, code, msg, error.info())
         except mechanize.HTTPError as exc:
-            self.assert_(exc is error)
+            self.assertTrue(exc is error)
         else:
-            self.assert_(False)
+            self.assertTrue(False)
 
     def test_robots(self):
         # XXX useragent
@@ -1169,7 +1173,7 @@ class HandlerTests(mechanize._testcase.TestCase):
         # first time: initialise and set up robots.txt parser before checking
         #  whether OK to fetch URL
         h.http_request(req)
-        self.assertEquals(rfpc.calls, [
+        self.assertEqual(rfpc.calls, [
             "__call__",
             ("set_opener", opener),
             ("set_url", "http://example.com:80/robots.txt"),
@@ -1181,7 +1185,7 @@ class HandlerTests(mechanize._testcase.TestCase):
         rfpc.clear()
         req = Request(url)
         h.http_request(req)
-        self.assert_(rfpc.calls == [
+        self.assertTrue(rfpc.calls == [
             ("can_fetch", "", url),
         ])
         # different URL on same server: same again
@@ -1189,7 +1193,7 @@ class HandlerTests(mechanize._testcase.TestCase):
         url = "http://example.com:80/blah.html"
         req = Request(url)
         h.http_request(req)
-        self.assert_(rfpc.calls == [
+        self.assertTrue(rfpc.calls == [
             ("can_fetch", "", url),
         ])
         # disallowed URL
@@ -1200,8 +1204,8 @@ class HandlerTests(mechanize._testcase.TestCase):
         try:
             h.http_request(req)
         except mechanize.HTTPError as e:
-            self.assert_(e.request == req)
-            self.assert_(e.code == 403)
+            self.assertTrue(e.request == req)
+            self.assertTrue(e.code == 403)
         # new host: reload robots.txt (even though the host and port are
         #  unchanged, we treat this as a new host because
         #  "example.com" != "example.com:80")
@@ -1210,7 +1214,7 @@ class HandlerTests(mechanize._testcase.TestCase):
         url = "http://example.com/rhubarb.html"
         req = Request(url)
         h.http_request(req)
-        self.assertEquals(rfpc.calls, [
+        self.assertEqual(rfpc.calls, [
             "__call__",
             ("set_opener", opener),
             ("set_url", "http://example.com/robots.txt"),
@@ -1223,7 +1227,7 @@ class HandlerTests(mechanize._testcase.TestCase):
         url = "https://example.org/rhubarb.html"
         req = Request(url)
         h.http_request(req)
-        self.assertEquals(rfpc.calls, [
+        self.assertEqual(rfpc.calls, [
             "__call__",
             ("set_opener", opener),
             ("set_url", "https://example.org/robots.txt"),
@@ -1236,29 +1240,26 @@ class HandlerTests(mechanize._testcase.TestCase):
         url = "ftp://example.com/"
         req = Request(url)
         h.http_request(req)
-        self.assert_(rfpc.calls == [])
+        self.assertTrue(rfpc.calls == [])
 
     def test_redirected_robots_txt(self):
         # redirected robots.txt fetch shouldn't result in another attempted
         # robots.txt fetch to check the redirection is allowed!
         import mechanize
-        from mechanize import build_opener, HTTPHandler, \
-            HTTPDefaultErrorHandler, HTTPRedirectHandler, \
-            HTTPRobotRulesProcessor
+        from mechanize import (
+                HTTPDefaultErrorHandler, HTTPRedirectHandler,
+                HTTPRobotRulesProcessor)
 
         class MockHTTPHandler(mechanize.BaseHandler):
             def __init__(self):
                 self.requests = []
 
             def http_open(self, req):
-                import mimetools
-                import httplib
                 import copy
-                from StringIO import StringIO
                 self.requests.append(copy.deepcopy(req))
                 if req.get_full_url() == "http://example.com/robots.txt":
-                    hdr = "Location: http://example.com/en/robots.txt\r\n\r\n"
-                    msg = mimetools.Message(StringIO(hdr))
+                    hdr = b"Location: http://example.com/en/robots.txt\r\n\r\n"
+                    msg = create_response_info(BytesIO(hdr))
                     return self.parent.error("http", req,
                                              test_response(), 302, "Blah", msg)
                 else:
@@ -1279,13 +1280,13 @@ class HandlerTests(mechanize._testcase.TestCase):
     def test_cookies(self):
         cj = MockCookieJar()
         h = HTTPCookieProcessor(cj)
-        o = h.parent = MockOpener()
+        h.parent = MockOpener()
 
         req = Request("http://example.com/")
         r = MockResponse(200, "OK", {}, "")
         newreq = h.http_request(req)
         self.assertTrue(cj.ach_req is req is newreq)
-        self.assertEquals(req.get_origin_req_host(), "example.com")
+        self.assertEqual(req.get_origin_req_host(), "example.com")
         self.assertFalse(cj.ach_u)
         newr = h.http_response(req, r)
         self.assertTrue(cj.ec_req is req)
@@ -1294,11 +1295,11 @@ class HandlerTests(mechanize._testcase.TestCase):
 
     def test_http_equiv(self):
         h = HTTPEquivProcessor()
-        o = h.parent = MockOpener()
+        h.parent = MockOpener()
 
-        data = ('<html><head>'
-                '<meta http-equiv="Refresh" content="spam&amp;eggs">'
-                '</head></html>')
+        data = ('<html><HEad>'
+                '<Meta httP-equiv="RefResh" coNtent="spam&amp;Eggs">'
+                '</Head></html>')
         headers = [
             ("Foo", "Bar"),
             ("Content-type", "text/html"),
@@ -1311,9 +1312,9 @@ class HandlerTests(mechanize._testcase.TestCase):
 
         new_headers = newr.info()
         self.assertEqual(new_headers["Foo"], "Bar")
-        self.assertEqual(new_headers["Refresh"], "spam&eggs")
+        self.assertEqual(new_headers["Refresh"], "spam&Eggs")
         self.assertEqual(
-            new_headers.getheaders("Refresh"), ["blah", "spam&eggs"])
+            new_headers.getheaders("Refresh"), ["blah", "spam&Eggs"])
 
     def test_refresh(self):
         # XXX test processor constructor optional args
@@ -1329,7 +1330,7 @@ class HandlerTests(mechanize._testcase.TestCase):
             req = Request("http://example.com/")
             headers = http_message({"refresh": val})
             r = MockResponse(200, "OK", headers, "", "http://example.com/")
-            newr = h.http_response(req, r)
+            h.http_response(req, r)
             if valid:
                 self.assertEqual(o.proto, "http")
                 self.assertEqual(o.args, (req, r, "refresh", "OK", headers))
@@ -1338,7 +1339,7 @@ class HandlerTests(mechanize._testcase.TestCase):
         class SleepTester:
             def __init__(self, test, seconds):
                 self._test = test
-                if seconds is 0:
+                if seconds == 0:
                     seconds = None  # don't expect a sleep for 0 seconds
                 self._expected = seconds
                 self._got = None
@@ -1479,7 +1480,6 @@ class HandlerTests(mechanize._testcase.TestCase):
     def test_refresh_bad_uri(self):
         # bad URIs should be cleaned up before redirection
         from mechanize._response import test_html_response
-        from_url = "http://example.com/a.html"
         bad_to_url = "http://example.com/b. |html"
         good_to_url = "http://example.com/b.%20%7Chtml"
 
@@ -1489,18 +1489,17 @@ class HandlerTests(mechanize._testcase.TestCase):
         req = Request("http://example.com/")
         r = test_html_response(
             headers=[("refresh", '0; url="%s"' % bad_to_url)])
-        newr = h.http_response(req, r)
+        h.http_response(req, r)
         headers = o.args[-1]
         self.assertEqual(headers["Location"], good_to_url)
 
     def test_cookie_redirect(self):
         # cookies shouldn't leak into redirected requests
-        import mechanize
-        from mechanize import CookieJar, build_opener, HTTPHandler, \
-            HTTPCookieProcessor, HTTPError, HTTPDefaultErrorHandler, \
-            HTTPRedirectHandler
+        from mechanize import (
+                CookieJar, HTTPCookieProcessor, HTTPDefaultErrorHandler,
+                HTTPRedirectHandler)
 
-        from test_cookies import interact_netscape
+        from test.test_cookies import interact_netscape
 
         cj = CookieJar()
         interact_netscape(cj, "http://www.example.com/", "spam=eggs")
@@ -1523,7 +1522,7 @@ class HandlerTests(mechanize._testcase.TestCase):
 
         req = Request("http://acme.example.com/")
         self.assertEqual(req.get_host(), "acme.example.com")
-        r = o.open(req)
+        o.open(req)
         self.assertEqual(req.get_host(), "proxy.example.com:3128")
 
         self.assertEqual([(handlers[0], "http_open")],
@@ -1536,11 +1535,11 @@ class HandlerTests(mechanize._testcase.TestCase):
         o.add_handler(ph)
         req = Request("http://www.perl.org/")
         self.assertEqual(req.get_host(), "www.perl.org")
-        r = o.open(req)
+        o.open(req)
         self.assertEqual(req.get_host(), "proxy.example.com")
         req = Request("http://www.python.org")
         self.assertEqual(req.get_host(), "www.python.org")
-        r = o.open(req)
+        o.open(req)
         if sys.version_info >= (2, 6):
             # no_proxy environment variable not supported in python 2.5
             self.assertEqual(req.get_host(), "www.python.org")
@@ -1573,7 +1572,7 @@ class HandlerTests(mechanize._testcase.TestCase):
         handlers = add_ordered_mock_handlers(o, meth_spec)
         req = Request("https://www.example.com/")
         self.assertEqual(req.get_host(), "www.example.com")
-        r = o.open(req)
+        o.open(req)
         self.assertEqual(req.get_host(), "proxy.example.com:3128")
         self.assertEqual([(handlers[0], "https_open")],
                          [tup[0:2] for tup in o.calls])
@@ -1633,14 +1632,15 @@ class HandlerTests(mechanize._testcase.TestCase):
         req.add_header("User-Agent", "Grail")
         self.assertEqual(req.get_host(), "www.example.com")
         self.assertIsNone(req._tunnel_host)
-        r = o.open(req)
+        o.open(req)
         # Verify Proxy-Authorization gets tunneled to request.
         # httpsconn req_headers do not have the Proxy-Authorization header but
         # the req will have.
-        self.assertFalse(("Proxy-Authorization",
-                          "FooBar") in https_handler.httpconn.req_headers)
-        self.assertTrue(
-            ("User-Agent", "Grail") in https_handler.httpconn.req_headers)
+        self.assertNotIn(
+            ("Proxy-Authorization", "FooBar"),
+            https_handler.httpconn.req_headers)
+        self.assertIn(
+            ("User-Agent", "Grail"), https_handler.httpconn.req_headers)
         self.assertIsNotNone(req._tunnel_host)
         self.assertEqual(req.get_host(), "proxy.example.com:3128")
         self.assertEqual(req.get_header("Proxy-authorization"), "FooBar")
@@ -1713,7 +1713,7 @@ class HandlerTests(mechanize._testcase.TestCase):
         self.assertEqual(user, password_manager.user)
         self.assertEqual(password, password_manager.password)
 
-        r = opener.open(request_url)
+        opener.open(request_url)
 
         # should have asked the password manager for the username/password
         self.assertEqual(password_manager.target_realm, realm)
@@ -1722,16 +1722,16 @@ class HandlerTests(mechanize._testcase.TestCase):
         # expect one request without authorization, then one with
         self.assertEqual(len(http_handler.requests), 2)
         self.assertFalse(http_handler.requests[0].has_header(auth_header))
-        userpass = '%s:%s' % (user, password)
-        auth_hdr_value = 'Basic ' + base64.encodestring(userpass).strip()
+        userpass = ('%s:%s' % (user, password)).encode('utf-8')
+        auth_hdr_value = b'Basic ' + base64.b64encode(userpass).strip()
         self.assertEqual(http_handler.requests[1].get_header(auth_header),
-                         auth_hdr_value)
+                         auth_hdr_value.decode('ascii'))
 
         # if the password manager can't find a password, the handler won't
         # handle the HTTP auth error
         password_manager.user = password_manager.password = None
         http_handler.reset()
-        r = opener.open(request_url)
+        opener.open(request_url)
         self.assertEqual(len(http_handler.requests), 1)
         self.assertFalse(http_handler.requests[0].has_header(auth_header))
 
@@ -1742,7 +1742,7 @@ class HeadParserTests(unittest.TestCase):
         htmls = [
             (
                 b"""<meta http-equiv=refresh content="1; http://example.com/">
-                """, [("refresh", "1; http://example.com/")]),
+                """, [(b"refresh", b"1; http://example.com/")]),
 
             (
                 b"""
@@ -1754,7 +1754,10 @@ class HeadParserTests(unittest.TestCase):
                 <meta http-equiv="moo" content="cow">
                 </html>
                 """,
-                [("refresh", "1; http://example.com/"), ("f\noo", "b\\ar")]),
+                [
+                    (b"refresh", b"1; http://example.com/"),
+                    (b"f\noo", b"b\\ar")
+                ]),
 
             (
                 b"""<meta http-equiv="refresh">
